# Two stage example Virtual Machine file
# usb port needs to be set in initInterfaces
# Nadya Peek Dec 2014

#------IMPORTS-------

#------Pygestalt-------
from pygestalt import nodes
from pygestalt import interfaces
from pygestalt import machines
from pygestalt import functions
from pygestalt.machines import elements
from pygestalt.machines import kinematics
from pygestalt.machines import state
from pygestalt.utilities import notice
from pygestalt.publish import rpc       #remote procedure call dispatcher

#------Other Libraries-------
import time
import io
import sys
import wx
import wx.lib.platebtn as platebtn
import csv

#------VIRTUAL MACHINE------
class virtualMachine(machines.virtualMachine):

    def initInterfaces(self):
        if self.providedInterface: self.fabnet = self.providedInterface         #providedInterface is defined in the virtualMachine class.
        else: self.fabnet = interfaces.gestaltInterface('FABNET', interfaces.serialInterface(baudRate = 115200, interfaceType = 'ftdi', portName = 'COM67'))

    def initControllers(self):
        self.yAxisNode = nodes.networkedGestaltNode('Y Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
        self.zlAxisNode = nodes.networkedGestaltNode('ZL Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
        self.zrAxisNode = nodes.networkedGestaltNode('ZR Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
        self.wAxisNode = nodes.networkedGestaltNode('W Axis', self.fabnet, filename = '086-005a.py', persistence = self.persistence)
        self.yzzwNode = nodes.compoundNode(self.yAxisNode, self.zlAxisNode, self.zrAxisNode, self.wAxisNode)

    def initCoordinates(self):
        self.position = state.coordinate(['mm', 'mm', 'mm', 'rev'])

    def initKinematics(self):
        self.yAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(8), elements.invert.forward(False)])
        self.zlAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(8), elements.invert.forward(False)])
        self.zrAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8), elements.leadscrew.forward(8), elements.invert.forward(False)])
        self.wAxis = elements.elementChain.forward([elements.microstep.forward(4), elements.stepper.forward(1.8),  elements.invert.forward(False)])

        self.stageKinematics = kinematics.direct(4)     #direct drive on all axes

    def initFunctions(self):
        self.move = functions.move(virtualMachine = self, virtualNode = self.yzzwNode, axes = [self.yAxis, self.zlAxis, self.zrAxis, self.wAxis], kinematics = self.stageKinematics, machinePosition = self.position,planner = 'null')
        self.jog = functions.jog(self.move)     #an incremental wrapper for the move function
        pass

    def initLast(self):
        #self.machineControl.setMotorCurrents(aCurrent = 0.8, bCurrent = 0.8, cCurrent = 0.8)
        #self.xNode.setVelocityRequest(0)       #clear velocity on nodes. Eventually this will be put in the motion planner on initialization to match state.
        pass

    def publish(self):
        #self.publisher.addNodes(self.machineControl)
        pass

    def getPosition(self):
        return {'position':self.position.future()}

    def setPosition(self, position  = [None]):
        self.position.future.set(position)

    def setSpindleSpeed(self, speedFraction):
        #self.machineControl.pwmRequest(speedFraction)
        pass


#Funtions
def openFile():
    fname = raw_input("Enter filename: ")
    if len(fname) == 0:
        fname = "test2.csv"
    try:
        fh = open(fname,'r')
    except:
        print "Error: File (", fname, ") cannot be opened. \nOpening test file instead."
        fname = "test2.csv"
        fh = open(fname,'r')
        #exit()

    circcoords = []
    for line in fh.readlines():
        xy = line.split(',')
        coord = []
        for num in xy:
            try:
                nr = int(num)
                coord.append(nr)
            except:
                pass
        circcoords.append(coord)
    circcoords.append([0,0,0,0])
    fh.close()

    move_Motors(circcoords)
    #print circcoords
    print ("Reading done!")

def move_Motors(moves):


    for element in moves:
        print element

    print ("Loop Done")
    '''
    for move in moves:
        stages.move(move, 0)
        status = stages.yAxisNode.spinStatusRequest()
        # This checks to see if the move is done.
        while status['stepsRemaining'] > 0:
            time.sleep(0.001)
            status = stages.yAxisNode.spinStatusRequest()
    '''


def machineSetup():
    # The persistence file remembers the node you set. It'll generate the first time you run the
    # file. If you are hooking up a new node, delete the previous persistence file.
    stages = virtualMachine(persistenceFile = "test.vmp")

    # You can load a new program onto the nodes if you are so inclined. This is currently set to
    # the path to the 086-005 repository on Nadya's machine.
    #stages.xyNode.loadProgram('../../../086-005/086-005a.hex')


    # This is a widget for setting the potentiometer to set the motor current limit on the nodes.
    # The A4982 has max 2A of current, running the widget will interactively help you set.
    #stages.xyNode.setMotorCurrent(0.7)

    # This is for how fast the motors for move
    stages.yzzwNode.setVelocityRequest(1)

class MyPanel(wx.Panel):
    def __init__(self, parent):
        super(MyPanel, self).__init__(parent)

        # Sizer to control button layout
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        vsizer = wx.BoxSizer(wx.VERTICAL)
        vsizer.Add(sizer)

        # Button with a bitmap
        button = wx.Button(self, label="Open File")
        bitmap = wx.Bitmap('imgs/073.png')
        button.SetBitmap(bitmap)
        sizer.Add(button)

        # Button with a bitmap
        buttoncut = wx.Button(self, label="Cut")
        bitmap = wx.Bitmap('imgs/043.png')
        buttoncut.SetBitmap(bitmap)
        sizer.Add(buttoncut)

        # Button with a bitmap
        buttonabout = wx.Button(self, label="About")
        bitmap = wx.Bitmap('imgs/021.png')
        buttonabout.SetBitmap(bitmap)
        sizer.Add(buttonabout)

        self.SetSizer(sizer)
        self.Bind(wx.EVT_BUTTON, self.OnButton)

    def OnButton(self,event):
        buttonLabel =  event.GetEventObject().GetLabel()
        if buttonLabel == "Open File":
            print ("Please enter the name of the file to open")
            openFile()
            self.GetParent().SetStatusText('Opening File')
        elif buttonLabel == "About":
            print ("About")
            self.GetParent().SetStatusText('Know about us')
            self.Bind(wx.EVT_BUTTON, self.GetParent().OnAbout, id=wx.ID_ABOUT)


class MyFrame(wx.Frame):
    def __init__(self, parent, title=""):
        super(MyFrame, self).__init__(parent, title=title)

        # Set the panel
        self.panel = MyPanel(self)

        # Setup Menus
        menubar = wx.MenuBar()
        helpmenu = wx.Menu()
        helpmenu.Append(wx.ID_ABOUT, "About")
        menubar.Append(helpmenu, "Help")
        self.SetMenuBar(menubar)

        # Setup StatusBar
        self.CreateStatusBar()
        self.PushStatusText("Run the Machine configuration")

        # Event Handlers
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)

    #About menu
    def OnAbout(self, event):
        """Show the about dialog"""
        info = wx.AboutDialogInfo()

        # Make a template for the description
        desc = ["\nFoam Cutter 2016\n",
                "Platform Info: (%s,%s)",
                "License: Public Domain"]
        desc = "\n".join(desc)



        # Populate with information
        info.SetName("Foldable Mcnulty - FA16")
        info.SetVersion("0.5")
        info.SetCopyright("Copyright (C) Luis Diaz, Borja Lanza, Marta Verde\nPaco Gonzalez, Jose Real, Maria Santisteban, Victoria Retana")
        info.SetWebSite("http://archive.fabacademy.org/archives/2016/fablabmadridceu/mtm/mtm2016.html")
        info.SetDescription("Foam Cutter GUI system")

        # Create and show the dialog
        wx.AboutBox(info)



class MyApp(wx.App):
    def OnInit(self):
        #machineSetup()

        self.frame = MyFrame(None, title="McNulty Foldable Hot Wire Foam Machine")
        self.frame.Show();
        return True


if __name__ == "__main__":
    app = MyApp(False)
    app.MainLoop()